# Copyright (c) 2022, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import os
import typing as t

from ansible.module_utils.common.collections import is_sequence
from ansible.module_utils.common.locale import get_best_parsable_locale

from ansible_collections.community.general.plugins.module_utils import _cmd_runner_fmt as cmd_runner_fmt

if t.TYPE_CHECKING:
    from collections.abc import Callable, Mapping, Sequence

    from ansible.module_utils.basic import AnsibleModule

    from ansible_collections.community.general.plugins.module_utils._cmd_runner_fmt import ArgFormatType

    ArgFormatter = t.Union[ArgFormatType, cmd_runner_fmt._ArgFormat]  # noqa: UP007


def _ensure_list(value):
    return list(value) if is_sequence(value) else [value]


def _process_as_is(rc, out, err):
    return rc, out, err


class CmdRunnerException(Exception):
    pass


class MissingArgumentFormat(CmdRunnerException):
    def __init__(self, arg, args_order: tuple[str, ...], args_formats) -> None:
        self.args_order = args_order
        self.arg = arg
        self.args_formats = args_formats

    def __repr__(self):
        return f"MissingArgumentFormat({self.arg!r}, {self.args_order!r}, {self.args_formats!r})"

    def __str__(self):
        return f"Cannot find format for parameter {self.arg} {self.args_order} in: {self.args_formats}"


class MissingArgumentValue(CmdRunnerException):
    def __init__(self, args_order: tuple[str, ...], arg) -> None:
        self.args_order = args_order
        self.arg = arg

    def __repr__(self):
        return f"MissingArgumentValue({self.args_order!r}, {self.arg!r})"

    def __str__(self):
        return f"Cannot find value for parameter {self.arg} in {self.args_order}"


class FormatError(CmdRunnerException):
    def __init__(self, name, value, args_formats, exc):
        self.name = name
        self.value = value
        self.args_formats = args_formats
        self.exc = exc
        super().__init__()

    def __repr__(self):
        return f"FormatError({self.name!r}, {self.value!r}, {self.args_formats!r}, {self.exc!r})"

    def __str__(self):
        return f"Failed to format parameter {self.name} with value {self.value}: {self.exc}"


class CmdRunner:
    """
    Wrapper for ``AnsibleModule.run_command()``.

    It aims to provide a reusable runner with consistent argument formatting
    and sensible defaults.
    """

    @staticmethod
    def _prepare_args_order(order: str | Sequence[str]) -> tuple[str, ...]:
        return tuple(order) if is_sequence(order) else tuple(order.split())  # type: ignore

    def __init__(
        self,
        module: AnsibleModule,
        command,
        arg_formats: Mapping[str, ArgFormatter] | None = None,
        default_args_order: str | Sequence[str] = (),
        check_rc: bool = False,
        force_lang: str = "C",
        path_prefix: Sequence[str] | None = None,
        environ_update: dict[str, str] | None = None,
    ):
        self.module = module
        self.command = _ensure_list(command)
        self.default_args_order = self._prepare_args_order(default_args_order)
        if arg_formats is None:
            arg_formats = {}
        self.arg_formats = {}
        for fmt_name, fmt in arg_formats.items():
            if not cmd_runner_fmt.is_argformat(fmt):
                fmt = cmd_runner_fmt.as_func(func=fmt, ignore_none=True)
            self.arg_formats[fmt_name] = fmt
        self.check_rc = check_rc
        if force_lang == "auto":
            try:
                self.force_lang = get_best_parsable_locale(module)
            except RuntimeWarning:
                self.force_lang = "C"
        else:
            self.force_lang = force_lang
        self.path_prefix = path_prefix
        if environ_update is None:
            environ_update = {}
        self.environ_update = environ_update

        _cmd = self.command[0]
        self.command[0] = (
            _cmd
            if (os.path.isabs(_cmd) or "/" in _cmd)
            else module.get_bin_path(_cmd, opt_dirs=path_prefix, required=True)
        )

    @property
    def binary(self) -> str:
        return self.command[0]

    def __call__(
        self,
        args_order: str | Sequence[str] | None = None,
        output_process: Callable[[int, str, str], t.Any] | None = None,
        check_mode_skip: bool = False,
        check_mode_return: t.Any | None = None,
        **kwargs,
    ):
        if output_process is None:
            output_process = _process_as_is
        if args_order is None:
            args_order = self.default_args_order
        args_order = self._prepare_args_order(args_order)
        for p in args_order:
            if p not in self.arg_formats:
                raise MissingArgumentFormat(p, args_order, tuple(self.arg_formats.keys()))
        return _CmdRunnerContext(
            runner=self,
            args_order=args_order,
            output_process=output_process,
            check_mode_skip=check_mode_skip,
            check_mode_return=check_mode_return,
            **kwargs,
        )

    def has_arg_format(self, arg):
        return arg in self.arg_formats

    # not decided whether to keep it or not, but if deprecating it will happen in a farther future.
    context = __call__


class _CmdRunnerContext:
    def __init__(
        self,
        runner: CmdRunner,
        args_order: tuple[str, ...],
        output_process: Callable[[int, str, str], t.Any],
        check_mode_skip: bool,
        check_mode_return: t.Any,
        **kwargs,
    ) -> None:
        self.runner = runner
        self.args_order = tuple(args_order)
        self.output_process = output_process
        self.check_mode_skip = check_mode_skip
        self.check_mode_return = check_mode_return
        self.run_command_args = dict(kwargs)

        self.environ_update = runner.environ_update
        self.environ_update.update(self.run_command_args.get("environ_update", {}))
        if runner.force_lang:
            self.environ_update.update(
                {
                    "LANGUAGE": runner.force_lang,
                    "LC_ALL": runner.force_lang,
                }
            )
        self.run_command_args["environ_update"] = self.environ_update

        if "check_rc" not in self.run_command_args:
            self.run_command_args["check_rc"] = runner.check_rc
        self.check_rc = self.run_command_args["check_rc"]

        self.cmd = None
        self.results_rc = None
        self.results_out = None
        self.results_err = None
        self.results_processed = None

    def run(self, **kwargs):
        runner = self.runner
        module = self.runner.module
        self.cmd = list(runner.command)
        self.context_run_args = dict(kwargs)

        named_args = dict(module.params)
        named_args.update(kwargs)
        for arg_name in self.args_order:
            value = None
            try:
                if arg_name in named_args:
                    value = named_args[arg_name]
                elif not runner.arg_formats[arg_name].ignore_missing_value:
                    raise MissingArgumentValue(self.args_order, arg_name)
                self.cmd.extend(runner.arg_formats[arg_name](value))
            except MissingArgumentValue:
                raise
            except Exception as e:
                raise FormatError(arg_name, value, runner.arg_formats[arg_name], e) from e

        if self.check_mode_skip and module.check_mode:
            return self.check_mode_return
        results = module.run_command(self.cmd, **self.run_command_args)
        self.results_rc, self.results_out, self.results_err = results
        self.results_processed = self.output_process(*results)
        return self.results_processed

    @property
    def run_info(self) -> dict[str, t.Any]:
        return dict(
            check_rc=self.check_rc,
            environ_update=self.environ_update,
            args_order=self.args_order,
            cmd=self.cmd,
            run_command_args=self.run_command_args,
            context_run_args=self.context_run_args,
            results_rc=self.results_rc,
            results_out=self.results_out,
            results_err=self.results_err,
            results_processed=self.results_processed,
        )

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        return False



# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
"""
The eos interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

import re

from copy import deepcopy

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.interfaces.interfaces import (
    InterfacesArgs,
)


class InterfacesFacts(object):
    """The eos interfaces fact class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = InterfacesArgs.argument_spec
        spec = deepcopy(self.argument_spec)
        if subspec:
            if options:
                facts_argument_spec = spec[subspec][options]
            else:
                facts_argument_spec = spec[subspec]
        else:
            facts_argument_spec = spec

        self.generated_spec = utils.generate_dict(facts_argument_spec)

    def get_device_data(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for interfaces

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected configuration
        :rtype: dictionary
        :returns: facts
        """
        if not data:
            data = self.get_device_data(connection)

        # operate on a collection of resource x
        config = ("\n" + data).split("\ninterface ")
        objs = []
        for conf in config:
            if conf:
                obj = self.render_config(self.generated_spec, conf)
                if obj:
                    objs.append(obj)
        facts = {"interfaces": []}
        if objs:
            params = utils.validate_config(
                self.argument_spec,
                {"config": objs},
            )
            for cfg in params["config"]:
                facts["interfaces"].append(utils.remove_empties(cfg))
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts

    def render_config(self, spec, conf):
        """
        Render config as dictionary structure and delete keys from spec for null values

        :param spec: The facts tree, generated from the argspec
        :param conf: The configuration
        :rtype: dictionary
        :returns: The generated config
        """
        config = deepcopy(spec)

        # populate the facts from the configuration
        config["name"] = re.match(r"(\S+)", conf).group(1)
        description = utils.parse_conf_arg(conf, "description")
        if description is not None:
            config["description"] = description.replace('"', "")
        shutdown = utils.parse_conf_cmd_arg(conf, "shutdown", False)
        config["enabled"] = shutdown if shutdown is False else True
        config["mtu"] = utils.parse_conf_arg(conf, "mtu")
        config["mode"] = utils.parse_conf_cmd_arg(
            conf,
            "switchport",
            "layer2",
            "layer3",
        )

        state = utils.parse_conf_arg(conf, "speed")
        if state:
            if state == "auto":
                config["duplex"] = state
            else:
                # remaining options are all e.g., 10half or 40gfull
                config["speed"] = state[:-4]
                config["duplex"] = state[-4:]
        return utils.remove_empties(config)



# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from copy import deepcopy

try:
    from botocore.exceptions import WaiterError
except ImportError:
    pass

from typing import Any
from typing import Dict
from typing import List
from typing import Optional
from typing import Tuple

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.ec2 import AnsibleEC2Error
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import create_transit_gateway_vpc_attachment
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import delete_transit_gateway_vpc_attachment
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_subnets
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import describe_transit_gateway_vpc_attachments
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import ensure_ec2_tags
from ansible_collections.amazon.aws.plugins.module_utils.ec2 import modify_transit_gateway_vpc_attachment
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_specifications
from ansible_collections.amazon.aws.plugins.module_utils.transformation import ansible_dict_to_boto3_filter_list
from ansible_collections.amazon.aws.plugins.module_utils.transformation import boto3_resource_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter


def get_states() -> List[str]:
    return [
        "available",
        "deleting",
        "failed",
        "failing",
        "initiatingRequest",
        "modifying",
        "pendingAcceptance",
        "pending",
        "rollingBack",
        "rejected",
        "rejecting",
    ]


def subnets_to_vpc(
    client, module: AnsibleAWSModule, subnets: List[str], subnet_details: Optional[List[Dict[str, Any]]] = None
) -> Optional[str]:
    if not subnets:
        return None

    if subnet_details is None:
        try:
            subnet_details = describe_subnets(client, SubnetIds=list(subnets))
        except AnsibleEC2Error as e:
            module.fail_json_aws_error(e)

    vpcs = [s.get("VpcId") for s in subnet_details]
    if len(set(vpcs)) > 1:
        module.fail_json(
            msg="Attachment subnets may only be in one VPC, multiple VPCs found",
            vpcs=list(set(vpcs)),
            subnets=subnet_details,
        )

    return vpcs[0]


def find_existing_attachment(
    client, module: AnsibleAWSModule, filters: Optional[Dict[str, Any]] = None, attachment_id: Optional[str] = None
) -> Optional[Dict[str, Any]]:
    """Find an existing transit gateway attachment based on filters or attachment ID.

    Args:
        client: The AWS client used to interact with the EC2 service.
        module: The Ansible module instance used for error handling.
        filters (Optional[Dict[str, Any]]): A dictionary of filters to apply when searching for attachments.
        attachment_id (Optional[str]): The ID of a specific attachment to find.

    Returns:
        Optional[Dict[str, Any]]: The found attachment details or None if not found.

    Raises:
        ValueError: If multiple attachments match the criteria.
    """
    # Find an existing attachment based on filters
    params = {}

    if attachment_id:
        params["TransitGatewayAttachmentIds"] = [attachment_id]
    elif filters:
        params["Filters"] = ansible_dict_to_boto3_filter_list(filters)

    try:
        attachments = describe_transit_gateway_vpc_attachments(client, **params)
    except AnsibleEC2Error as e:
        module.fail_json_aws_error(e)

    if len(attachments) > 1:
        raise ValueError("Multiple matching attachments found, provide an ID.")

    return attachments[0] if attachments else None


class TransitGatewayAttachmentStateManager:
    def __init__(self, client, module: AnsibleAWSModule, attachment_id: str) -> None:
        self.client = client
        self.module = module
        self.attachment_id = attachment_id

    @property
    def waiter_config(self) -> Dict[str, Any]:
        params: Dict[str, Any] = {}

        delay = min(5, self.module.params.get("wait_timeout"))
        max_attempts = self.module.params.get("wait_timeout") // delay
        config = dict(Delay=delay, MaxAttempts=max_attempts)
        params["WaiterConfig"] = config

        return params

    def create_attachment(self, params: Dict[str, Any]) -> str:
        """
        Create a new transit gateway attachment.

        Args:
            params (Dict[str, Any]): A dictionary containing the parameters needed to
                create the transit gateway attachment.

        Returns:
            str: The ID of the newly created transit gateway attachment.

        Raises:
            AnsibleEC2Error: If there is an error while creating the VPC attachment,
                it will fail the module and provide an error message.
        """
        try:
            tags = params.pop("Tags")
        except KeyError:
            tags = None

        if tags:
            params["TagSpecifications"] = boto3_tag_specifications(tags, types=["transit-gateway-attachment"])

        try:
            response = create_transit_gateway_vpc_attachment(self.client, **params)
        except AnsibleEC2Error as e:
            self.module.fail_json_aws_error(e)

        self.attachment_id = response["TransitGatewayAttachmentId"]

        return response["TransitGatewayAttachmentId"]

    def delete_attachment(self) -> bool:
        # Delete the transit gateway attachment

        if not self.attachment_id:
            return False

        if not self.module.check_mode:
            try:
                delete_transit_gateway_vpc_attachment(self.client, self.attachment_id)
            except AnsibleEC2Error as e:
                self.module.fail_json_aws_error(e)

        return True

    def wait_for_state_change(self, desired_state: str) -> None:
        # Wait until attachment reaches the desired state
        params = {"TransitGatewayAttachmentIds": [self.attachment_id]}
        params.update(self.waiter_config)
        waiter = get_waiter(self.client, f"transit_gateway_vpc_attachment_{desired_state}")

        try:
            waiter.wait(**params)
        except WaiterError as e:
            self.module.fail_json_aws(e, "Timeout waiting for State change")


class AttachmentConfigurationManager:
    def __init__(self, client, module: AnsibleAWSModule, attachment_id: str, existing: Dict[str, Any]) -> None:
        self.client = client
        self.module = module
        self.attachment_id = attachment_id

        self.existing = existing or {}
        self._resource_updates = {}
        self._subnets_to_add = []
        self._subnets_to_remove = []

    @property
    def resource_updates(self) -> Dict[str, Any]:
        return self._resource_updates

    @property
    def subnets_to_add(self) -> List[str]:
        return self._subnets_to_add

    @property
    def subnets_to_remove(self) -> List[str]:
        return self._subnets_to_remove

    def set_subnets(self, subnets: Optional[List[str]] = None, purge: bool = True) -> None:
        """
        Set or update the subnets associated with the transit gateway attachment.

        Args:
            subnets (Optional[List[str]]): A list of subnet IDs to associate with
                the attachment.
            purge (bool): If True, the existing subnets will be replaced with the
                specified subnets.
        """
        # Set or update the subnets associated with the attachment
        if subnets is None:
            return

        current_subnets = set(self.existing.get("SubnetIds", []))
        desired_subnets = set(subnets)
        if not purge:
            desired_subnets = desired_subnets.union(current_subnets)

        # We'll pull the VPC ID from the subnets, no point asking for
        # information we 'know'.
        try:
            subnet_details = describe_subnets(self.client, SubnetIds=list(desired_subnets))
        except AnsibleEC2Error as e:
            self.module.fail_json_aws_error(e)
        vpc_id = subnets_to_vpc(self.client, self.module, desired_subnets, subnet_details)
        self._set_resource_value("VpcId", vpc_id, immutable=True)

        # Only one subnet per-AZ is permitted
        azs = [s.get("AvailabilityZoneId") for s in subnet_details]
        if len(azs) != len(set(azs)):
            self.module.fail_json(
                msg="Only one attachment subnet per availability zone may be set.",
                availability_zones=azs,
                subnets=subnet_details,
            )

        self._subnets_to_add = list(desired_subnets.difference(current_subnets))
        self._subnets_to_remove = list(current_subnets.difference(desired_subnets))
        self._set_resource_value("SubnetIds", list(desired_subnets))

    def set_dns_support(self, value):
        return self._set_option("DnsSupport", value)

    def set_ipv6_support(self, value):
        return self._set_option("Ipv6Support", value)

    def set_appliance_mode_support(self, value):
        return self._set_option("ApplianceModeSupport", value)

    def set_transit_gateway(self, tgw_id: str):
        return self._set_resource_value("TransitGatewayId", tgw_id)

    def set_vpc(self, vpc_id: str):
        return self._set_resource_value("VpcId", vpc_id)

    def set_tags(self, tags, purge_tags):
        current_tags = boto3_tag_list_to_ansible_dict(self.existing.get("Tags", None))

        if purge_tags:
            desired_tags = deepcopy(tags)
        else:
            desired_tags = {**current_tags, **tags}

        self._set_resource_value("Tags", desired_tags)

    def _get_resource_value(self, key, default=None):
        default_value = self.existing.get(key, default)
        return self._resource_updates.get(key, default_value)

    def _set_option(self, name: str, value: Optional[bool]) -> bool:
        """
        Set a VPC attachment option to either enable or disable.

        Args:
            name (str): The name of the option to be updated.
            value (Optional[bool]): A boolean indicating whether to enable (True)
                or disable (False) the specified option. If None, no action is
                taken.

        Returns:
            bool: Returns True if the option was successfully set, or False if
            no update was made (because the value was None).
        """
        if value is None:
            return False

        # For now VPC Attachment options are all enable/disable
        value = "enable" if value else "disable"

        options = deepcopy(self.existing.get("Options", dict()))
        options.update(self._resource_updates.get("Options", dict()))
        options[name] = value

        return self._set_resource_value("Options", options)

    def _set_resource_value(self, key, value, description: Optional[str] = None, immutable: bool = False) -> bool:
        """
        Set a value for a resource attribute and track changes.

        Args:
            key (str): The attribute key to be updated.
            value (Any): The new value to set for the specified key.
            description (Optional[str], optional): A human-readable description of the
                resource attribute.
            immutable (bool, optional): A flag indicating whether the attribute is
                immutable. If True, and the resource exists, an error will be raised
                if attempting to change the value. Defaults to False.

        Returns:
            bool: Returns True if the value was successfully set, or False if no
            update was made.
        """
        if value is None or value == self._get_resource_value(key):
            return False

        if immutable and self.existing:
            description = description or key
            self.module.fail_json(msg=f"{description} can not be updated after creation")

        self.resource_updates[key] = value

        return True

    def filter_immutable_resource_attributes(self, resource: Dict[str, Any]) -> Dict[str, Any]:
        """
        Filter out immutable resource attributes from the given resource dictionary.

        Args:
            resource (Dict[str, Any]): A dictionary representing the resource, which
                may contain various attributes, including both mutable and immutable ones.

        Returns:
            Dict[str, Any]: A new dictionary containing only the mutable attributes
            of the resource.
        """
        immutable_options = ["TransitGatewayId", "VpcId", "VpcOwnerId", "State", "SubnetIds", "CreationTime", "Tags"]
        return {key: value for key, value in resource.items() if key not in immutable_options}


class TransitGatewayVpcAttachmentManager:
    def __init__(
        self, client, module: AnsibleAWSModule, existing: Dict[str, Any], attachment_id: Optional[str] = None
    ) -> None:
        self.client = client
        self.module = module
        self.attachment_id = attachment_id
        self.existing = existing or {}
        self.updated = {}
        self.changed = False

        self.state_manager = TransitGatewayAttachmentStateManager(client, module, attachment_id)
        self.config_manager = AttachmentConfigurationManager(client, module, attachment_id, existing)

    def merge_resource_changes(self, filter_immutable: bool = True) -> Dict[str, Any]:
        """Merge existing resource attributes with updates, optionally filtering out immutable attributes.

        Args:
            filter_immutable (bool): Whether to filter out immutable resource attributes. Defaults to True.

        Returns:
            Dict[str, Any]: The merged resource attributes.
        """
        resource = deepcopy(self.existing)
        resource.update(self.config_manager.resource_updates)

        if filter_immutable:
            resource = self.config_manager.filter_immutable_resource_attributes(resource)

        return resource

    def apply_configuration(self):
        """Apply configuration changes to the transit gateway attachment.

        Returns:
            bool: True if configuration changes were applied, False otherwise.
        """
        # Apply any configuration changes to the attachment
        if not self.attachment_id:
            return False

        updates = self.config_manager.filter_immutable_resource_attributes(self.config_manager.resource_updates)

        subnets_to_add = self.config_manager.subnets_to_add
        subnets_to_remove = self.config_manager.subnets_to_remove

        # Check if there are no changes to apply
        if not updates and not subnets_to_add and not subnets_to_remove:
            return False

        if subnets_to_add:
            updates["AddSubnetIds"] = subnets_to_add
        if subnets_to_remove:
            updates["RemoveSubnetIds"] = subnets_to_remove

        updates["TransitGatewayAttachmentId"] = self.attachment_id

        if not self.module.check_mode:
            try:
                modify_transit_gateway_vpc_attachment(self.client, **updates)
            except AnsibleEC2Error as e:
                self.module.fail_json_aws_error(e)
        return True

    def _set_configuration_parameters(self) -> None:
        """Set configuration parameters for the transit gateway attachment."""
        self.config_manager.set_transit_gateway(self.module.params.get("transit_gateway"))
        self.config_manager.set_subnets(self.module.params["subnets"], self.module.params.get("purge_subnets", True))
        self.config_manager.set_dns_support(self.module.params.get("dns_support"))
        self.config_manager.set_ipv6_support(self.module.params.get("ipv6_support"))
        self.config_manager.set_appliance_mode_support(self.module.params.get("appliance_mode_support"))

    def _prepare_tags(self) -> Tuple[Optional[Dict[str, str]], bool]:
        """Prepare and return the tags and purge flag.

        Returns:
            Tuple[Optional[Dict[str, str]], bool]: A tuple containing the tags dictionary and the purge flag.
        """
        tags = self.module.params.get("tags")
        purge_tags = self.module.params.get("purge_tags")

        if self.module.params.get("name"):
            new_tags = {"Name": self.module.params["name"]}
            if tags is None:
                purge_tags = False
            else:
                new_tags.update(tags)
            tags = new_tags

        return {} if tags is None else tags, purge_tags

    def _create_attachment(self) -> None:
        """Create a new transit gateway attachment."""
        if not self.module.check_mode:
            params = self.merge_resource_changes(filter_immutable=False)
            self.attachment_id = self.state_manager.create_attachment(params)

            if self.module.params.get("wait"):
                self.state_manager.wait_for_state_change("available")

        self.changed = True

    def _update_attachment(self, tags: Dict[str, Any], purge_tags: bool) -> None:
        """Update an existing transit gateway attachment."""
        if self.existing.get("State") == "pending":
            # Wait for resources to finish creating before updating
            self.state_manager.wait_for_state_change("available")
        elif self.existing.get("State") == "deleting":
            self.module.fail_json(
                msg="Deletion in progress, unable to update",
                attachments=[boto3_resource_to_ansible_dict(self.existing)],
            )

        # Apply the configuration
        if self.apply_configuration():
            self.changed = True
            if self.module.params.get("wait"):
                self.state_manager.wait_for_state_change("available")

        # Ensure tags are applied
        self.changed |= ensure_ec2_tags(
            self.client,
            self.module,
            self.attachment_id,
            resource_type="transit-gateway-attachment",
            tags=tags,
            purge_tags=purge_tags,
        )

    def create_or_modify_attachment(self):
        """Create or modify a transit gateway attachment based on the provided parameters."""

        # Set the configuration parameters
        self._set_configuration_parameters()

        # Handle tags
        tags, purge_tags = self._prepare_tags()

        # Set tags in the configuration manager
        self.config_manager.set_tags(tags, purge_tags)

        if not self.existing:
            self._create_attachment()
        else:
            self._update_attachment(tags, purge_tags)

        # Handle check mode updates
        if self.module.check_mode:
            self.updated = camel_dict_to_snake_dict(
                self.merge_resource_changes(filter_immutable=False), ignore_list=["Tags"]
            )
        else:
            self.updated = boto3_resource_to_ansible_dict(
                find_existing_attachment(self.client, self.module, attachment_id=self.attachment_id)
            )

    def delete_attachment(self):
        """Delete attachment"""
        if self.existing.get("State") == "deleting":
            if self.module.params.get("wait"):
                self.state_manager.wait_for_state_change("deleted")
            self.change = False
        else:
            self.changed |= self.state_manager.delete_attachment()
            if self.module.params.get("wait"):
                self.state_manager.wait_for_state_change("deleted")



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    # Standard files documentation fragment

    # NOTE: This document fragment needs to be augmented by ZONE_ID_TYPE in a provider document fragment.
    #       The ZONE_ID_TYPE fragment will provide `choices` for the options.type entry.
    DOCUMENTATION = r"""
description:
  - Records are matched by prefix / record name and value.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: full
options:
  state:
    description:
      - Specifies the state of the resource record.
    required: true
    choices: ['present', 'absent']
    type: str
  zone_name:
    description:
      - The DNS zone to modify.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
    type: str
    aliases:
      - zone
  zone_id:
    description:
      - The ID of the DNS zone to modify.
      - Exactly one of O(zone_name) and O(zone_id) must be specified.
  record:
    description:
      - The full DNS record to create or delete.
      - Exactly one of O(record) and O(prefix) must be specified.
    type: str
  prefix:
    description:
      - The prefix of the DNS record.
      - This is the part of O(record) before O(zone_name). For example, if the record to be modified is C(www.example.com)
        for the zone C(example.com), the prefix is V(www). If the record in this example would be C(example.com), the prefix
        would be V('') (empty string).
      - Exactly one of O(record) and O(prefix) must be specified.
    type: str
  ttl:
    description:
      - The TTL to give the new record, in seconds.
      - This is not used for record deletion.
    type: int
  type:
    description:
      - The type of DNS record to create or delete.
    required: true
    type: str
  value:
    description:
      - The new value when creating a DNS record.
      - When deleting a record all values for the record must be specified or it will not be deleted.
    required: true
    type: str
"""



# Copyright (c) 2016-2017, Yanis Guenane <yanis+ansible@guenane.org>
# Copyright (c) 2017, Markus Teufelberger <mteufelberger+ansible@mgit.at>
# Copyright (c) 2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t
from collections.abc import Callable

from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.math import (
    binary_exp_mod,
    quick_is_not_prime,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.module_backends.publickey_info import (
    _get_cryptography_public_key_info,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    get_fingerprint_of_bytes,
    load_privatekey,
)
from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover
    from cryptography.hazmat.primitives.asymmetric.types import (  # pragma: no cover
        PrivateKeyTypes,
    )

    from ansible_collections.community.crypto.plugins.plugin_utils._action_module import (  # pragma: no cover
        AnsibleActionModule,
    )
    from ansible_collections.community.crypto.plugins.plugin_utils._filter_module import (  # pragma: no cover
        FilterModuleMock,
    )

    GeneralAnsibleModule = t.Union[  # noqa: UP007
        AnsibleModule, AnsibleActionModule, FilterModuleMock
    ]  # pragma: no cover


MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    from cryptography.hazmat.primitives import serialization
except ImportError:
    pass

SIGNATURE_TEST_DATA = b"1234"


def _get_cryptography_private_key_info(
    key: PrivateKeyTypes, *, need_private_key_data: bool = False
) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
    key_type, key_public_data = _get_cryptography_public_key_info(key.public_key())
    key_private_data: dict[str, t.Any] = {}
    if need_private_key_data:
        if isinstance(key, cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey):
            rsa_private_numbers = key.private_numbers()
            key_private_data["p"] = rsa_private_numbers.p
            key_private_data["q"] = rsa_private_numbers.q
            key_private_data["exponent"] = rsa_private_numbers.d
        elif isinstance(
            key, cryptography.hazmat.primitives.asymmetric.dsa.DSAPrivateKey
        ):
            dsa_private_numbers = key.private_numbers()
            key_private_data["x"] = dsa_private_numbers.x
        elif isinstance(
            key, cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey
        ):
            ecc_private_numbers = key.private_numbers()
            key_private_data["multiplier"] = ecc_private_numbers.private_value
    return key_type, key_public_data, key_private_data


def _check_dsa_consistency(
    *, key_public_data: dict[str, t.Any], key_private_data: dict[str, t.Any]
) -> bool | None:
    # Get parameters
    p: int | None = key_public_data.get("p")
    if p is None:
        return None
    q: int | None = key_public_data.get("q")
    if q is None:
        return None
    g: int | None = key_public_data.get("g")
    if g is None:
        return None
    y: int | None = key_public_data.get("y")
    if y is None:
        return None
    x: int | None = key_private_data.get("x")
    if x is None:
        return None
    # Make sure that g is not 0, 1 or -1 in Z/pZ
    if g < 2 or g >= p - 1:
        return False
    # Make sure that x is in range
    if x < 1 or x >= q:
        return False
    # Check whether q divides p-1
    if (p - 1) % q != 0:
        return False
    # Check that g**q mod p == 1
    if binary_exp_mod(g, q, m=p) != 1:
        return False
    # Check whether g**x mod p == y
    if binary_exp_mod(g, x, m=p) != y:
        return False
    # Check (quickly) whether p or q are not primes
    return not (quick_is_not_prime(q) or quick_is_not_prime(p))


def _is_cryptography_key_consistent(
    key: PrivateKeyTypes,
    *,
    key_public_data: dict[str, t.Any],
    key_private_data: dict[str, t.Any],
    warn_func: Callable[[str], None] | None = None,
) -> bool | None:
    if isinstance(key, cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey):
        # key._backend was removed in cryptography 42.0.0
        backend = getattr(key, "_backend", None)
        if backend is not None:
            return bool(backend._lib.RSA_check_key(key._rsa_cdata))  # type: ignore  # pylint: disable=protected-access
    if isinstance(key, cryptography.hazmat.primitives.asymmetric.dsa.DSAPrivateKey):
        result = _check_dsa_consistency(
            key_public_data=key_public_data, key_private_data=key_private_data
        )
        if result is not None:
            return result
        signature = key.sign(
            SIGNATURE_TEST_DATA, cryptography.hazmat.primitives.hashes.SHA256()
        )
        try:
            key.public_key().verify(
                signature,
                SIGNATURE_TEST_DATA,
                cryptography.hazmat.primitives.hashes.SHA256(),
            )
            return True
        except cryptography.exceptions.InvalidSignature:
            return False
    if isinstance(
        key, cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey
    ):
        signature = key.sign(
            SIGNATURE_TEST_DATA,
            cryptography.hazmat.primitives.asymmetric.ec.ECDSA(
                cryptography.hazmat.primitives.hashes.SHA256()
            ),
        )
        try:
            key.public_key().verify(
                signature,
                SIGNATURE_TEST_DATA,
                cryptography.hazmat.primitives.asymmetric.ec.ECDSA(
                    cryptography.hazmat.primitives.hashes.SHA256()
                ),
            )
            return True
        except cryptography.exceptions.InvalidSignature:
            return False
    has_simple_sign_function = False
    if isinstance(
        key, cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey
    ):
        has_simple_sign_function = True
    if isinstance(key, cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey):
        has_simple_sign_function = True
    if has_simple_sign_function:
        signature = key.sign(SIGNATURE_TEST_DATA)  # type: ignore
        try:
            key.public_key().verify(signature, SIGNATURE_TEST_DATA)  # type: ignore
            return True
        except cryptography.exceptions.InvalidSignature:
            return False
    # For X25519 and X448, there's no test yet.
    if warn_func is not None:
        warn_func(f"Cannot determine consistency for key of type {type(key)}")
    return None


class PrivateKeyConsistencyError(OpenSSLObjectError):
    def __init__(self, msg: str, *, result: dict[str, t.Any]) -> None:
        super().__init__(msg)
        self.error_message = msg
        self.result = result


class PrivateKeyParseError(OpenSSLObjectError):
    def __init__(self, msg: str, *, result: dict[str, t.Any]) -> None:
        super().__init__(msg)
        self.error_message = msg
        self.result = result


class PrivateKeyInfoRetrieval:
    key: PrivateKeyTypes

    def __init__(
        self,
        *,
        module: GeneralAnsibleModule,
        content: bytes,
        passphrase: str | None = None,
        return_private_key_data: bool = False,
        check_consistency: bool = False,
    ):
        self.module = module
        self.content = content
        self.passphrase = passphrase
        self.return_private_key_data = return_private_key_data
        self.check_consistency = check_consistency

    def _get_public_key(self, *, binary: bool) -> bytes:
        return self.key.public_key().public_bytes(
            serialization.Encoding.DER if binary else serialization.Encoding.PEM,
            serialization.PublicFormat.SubjectPublicKeyInfo,
        )

    def _get_key_info(
        self, *, need_private_key_data: bool = False
    ) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
        return _get_cryptography_private_key_info(
            self.key, need_private_key_data=need_private_key_data
        )

    def _is_key_consistent(
        self, *, key_public_data: dict[str, t.Any], key_private_data: dict[str, t.Any]
    ) -> bool | None:
        return _is_cryptography_key_consistent(
            self.key,
            key_public_data=key_public_data,
            key_private_data=key_private_data,
            warn_func=self.module.warn,
        )

    def get_info(self, *, prefer_one_fingerprint: bool = False) -> dict[str, t.Any]:
        result: dict[str, t.Any] = {
            "can_parse_key": False,
            "key_is_consistent": None,
        }
        priv_key_detail = self.content
        try:
            self.key = load_privatekey(
                path=None,
                content=priv_key_detail,
                passphrase=(
                    to_bytes(self.passphrase)
                    if self.passphrase is not None
                    else self.passphrase
                ),
            )
            result["can_parse_key"] = True
        except OpenSSLObjectError as exc:
            raise PrivateKeyParseError(str(exc), result=result) from exc

        result["public_key"] = to_text(self._get_public_key(binary=False))
        pk = self._get_public_key(binary=True)
        result["public_key_fingerprints"] = (
            get_fingerprint_of_bytes(pk, prefer_one=prefer_one_fingerprint)
            if pk is not None
            else {}
        )

        key_type, key_public_data, key_private_data = self._get_key_info(
            need_private_key_data=self.return_private_key_data or self.check_consistency
        )
        result["type"] = key_type
        result["public_data"] = key_public_data
        if self.return_private_key_data:
            result["private_data"] = key_private_data

        if self.check_consistency:
            result["key_is_consistent"] = self._is_key_consistent(
                key_public_data=key_public_data, key_private_data=key_private_data
            )
            if result["key_is_consistent"] is False:
                # Only fail when it is False, to avoid to fail on None (which means "we do not know")
                msg = "Private key is not consistent! (See https://blog.hboeck.de/archives/888-How-I-tricked-Symantec-with-a-Fake-Private-Key.html)"
                raise PrivateKeyConsistencyError(msg, result=result)
        return result


def get_privatekey_info(
    *,
    module: GeneralAnsibleModule,
    content: bytes,
    passphrase: str | None = None,
    return_private_key_data: bool = False,
    prefer_one_fingerprint: bool = False,
) -> dict[str, t.Any]:
    info = PrivateKeyInfoRetrieval(
        module=module,
        content=content,
        passphrase=passphrase,
        return_private_key_data=return_private_key_data,
    )
    return info.get_info(prefer_one_fingerprint=prefer_one_fingerprint)


def select_backend(
    *,
    module: GeneralAnsibleModule,
    content: bytes,
    passphrase: str | None = None,
    return_private_key_data: bool = False,
    check_consistency: bool = False,
) -> PrivateKeyInfoRetrieval:
    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )
    return PrivateKeyInfoRetrieval(
        module=module,
        content=content,
        passphrase=passphrase,
        return_private_key_data=return_private_key_data,
        check_consistency=check_consistency,
    )


__all__ = (
    "PrivateKeyConsistencyError",
    "PrivateKeyInfoRetrieval",
    "PrivateKeyParseError",
    "get_privatekey_info",
    "select_backend",
)



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexiblebackup_info
version_added: "3.6.0"
short_description: Get Azure PostgreSQL Flexible Backup facts
description:
    - Get or list facts of PostgreSQL Flexible Backup.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        type: str
        required: True
    server_name:
        description:
            - The name of the post gresql server.
        type: str
        required: True
    backup_name:
        description:
            - The name of the post gresql backup.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - magodo (@magodo)
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: List instance of PostgreSQL Flexible Backup by server name
  azure_rm_postgresqlflexiblebackup_info:
    resource_group: myResourceGroup
    server_name: server_name

- name: Get instances of PostgreSQL Flexible Backup
  azure_rm_postgresqlflexiblebackup_info:
    resource_group: myResourceGroup
    server_name: server_name
    name: backup_name
'''

RETURN = '''
backup:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible Backup.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Fully qualified resource ID for the resource.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.DBforPostgreSQL/flexibleServers/posttest/backups/fredbackup"
        backup_name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: fredbackup
        server_name:
            description:
                - The post gresql flexible server name.
            returned: always
            type: str
            sample: posttest
        resource_group:
            description:
                - Name of the resource group.
            returned: always
            type: str
            sample: testRG
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: Microsoft.DBforPostgreSQL/flexibleServers/backups
        completed_time:
            description:
                - Backup completed time (ISO8601 format).
            type: str
            returned: always
            sample: "2025-04-17T08:11:58.756273+00:00"
        backup_type:
            description:
                - Backup type.
            type: str
            returned: always
            sample: Full
        source:
            description:
                - Backup source.
            type: str
            returned: always
            sample: Automatic
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMPostgreSqlFlexibleBackupInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            backup_name=dict(
                type='str'
            ),
        )
        # store the results of the module operation
        self.results = dict(
            changed=False
        )
        self.resource_group = None
        self.backup_name = None
        self.server_name = None
        super(AzureRMPostgreSqlFlexibleBackupInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.backup_name:
            self.results['backups'] = self.get()
        else:
            self.results['backups'] = self.list_all()
        return self.results

    def get(self):
        response = None
        results = []
        try:
            response = self.postgresql_flexible_client.backups_automatic_and_on_demand.get(resource_group_name=self.resource_group,
                                                                                           server_name=self.server_name,
                                                                                           backup_name=self.backup_name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError:
            self.log('Could not get backup facts for PostgreSQL Flexible Server.')

        if response:
            results.append(self.format_item(response))

        return results

    def list_all(self):
        response = None
        results = []
        try:
            response = self.postgresql_flexible_client.backups_automatic_and_on_demand.list_by_server(resource_group_name=self.resource_group,
                                                                                                      server_name=self.server_name)
            self.log("Response : {0}".format(response))
        except Exception as ec:
            self.log('Could not list backups facts for PostgreSQL Flexible Servers.')

        if response:
            for item in response:
                results.append(self.format_item(item))

        return results

    def format_item(self, item):
        return dict(
            resource_group=self.resource_group,
            server_name=self.server_name,
            backup_type=item.backup_type,
            completed_time=item.completed_time,
            id=item.id,
            name=item.name,
            source=item.source,
            type=item.type
        )


def main():
    AzureRMPostgreSqlFlexibleBackupInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The action plugin file for validate
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

from ansible.errors import AnsibleActionFail, AnsibleError
from ansible.module_utils.common.text.converters import to_text
from ansible.plugins.action import ActionBase

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    check_argspec,
)
from ansible_collections.ansible.utils.plugins.modules.validate import DOCUMENTATION
from ansible_collections.ansible.utils.plugins.plugin_utils.base.validate import _load_validator


ARGSPEC_CONDITIONALS = {}


class ActionModule(ActionBase):
    """action module"""

    VALIDATE_CLS_NAME = "Validate"
    _requires_connection = False

    def __init__(self, *args, **kwargs):
        super(ActionModule, self).__init__(*args, **kwargs)
        self._validator_name = None
        self._result = {}

    def _debug(self, name, msg):
        """Output text using ansible's display

        :param msg: The message
        :type msg: str
        """
        msg = "<{phost}> {name} {msg}".format(phost=self._playhost, name=name, msg=msg)
        self._display.vvvv(msg)

    def run(self, tmp=None, task_vars=None):
        """The std execution entry pt for an action plugin

        :param tmp: no longer used
        :type tmp: none
        :param task_vars: The vars provided when the task is run
        :type task_vars: dict
        :return: The results from the parser
        :rtype: dict
        """
        valid, argspec_result, updated_params = check_argspec(
            DOCUMENTATION,
            "validate module",
            schema_conditionals=ARGSPEC_CONDITIONALS,
            **self._task.args,
        )
        if not valid:
            return argspec_result

        self._task_vars = task_vars
        self._playhost = task_vars.get("inventory_hostname") if task_vars else None

        self._validator_engine, validator_result = _load_validator(
            engine=updated_params["engine"],
            data=updated_params["data"],
            criteria=updated_params["criteria"],
            plugin_vars=task_vars,
        )
        if validator_result.get("failed"):
            return validator_result

        try:
            result = self._validator_engine.validate()
        except AnsibleError as exc:
            raise AnsibleActionFail(to_text(exc, errors="surrogate_then_replace"))
        except Exception as exc:
            raise AnsibleActionFail(
                "Unhandled exception from validator '{validator}'. Error: {err}".format(
                    validator=self._validator_engine,
                    err=to_text(exc, errors="surrogate_then_replace"),
                ),
            )

        self._result["msg"] = ""
        if result.get("errors"):
            self._result["errors"] = result["errors"]
            self._result.update({"failed": True})
            if "msg" in result:
                self._result["msg"] = "Validation errors were found.\n" + result["msg"]
            else:
                self._result["msg"] = "Validation errors were found."

        if result.get("warnings", []):
            self._result["warnings"] = result["warnings"]
            if not self._result["msg"]:
                self._result["msg"] = "Non-fatal validation errors were found."

        if not self._result["msg"]:
            self._result["msg"] = "All checks passed."

        return self._result



#!/usr/bin/python

# Copyright (c) 2025, Shahar Golshani (@shahargolshani)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: file_remove

short_description: Remove files matching a pattern from a directory

description:
  - This module removes files from a specified directory that match a given pattern.
    The pattern can include wildcards and regular expressions.
  - By default, only files in the specified directory are removed (non-recursive).
    Use the O(recursive) option to search and remove files in subdirectories.

version_added: "12.1.0"

author:
  - Shahar Golshani (@shahargolshani)

extends_documentation_fragment:
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: full

options:
  path:
    description:
      - Path to the directory where files should be removed.
      - This must be an existing directory.
    type: path
    required: true

  pattern:
    description:
      - Pattern to match files for removal.
      - Supports wildcards (V(*), V(?), V([seq]), V([!seq])) for glob-style matching.
      - Use O(use_regex=true) to interpret this as a regular expression instead.
    type: str
    required: true

  use_regex:
    description:
      - If V(true), O(pattern) is interpreted as a regular expression.
      - If V(false), O(pattern) is interpreted as a glob-style wildcard pattern.
    type: bool
    default: false

  recursive:
    description:
      - If V(true), search for files recursively in subdirectories.
      - If V(false), only files in the specified directory are removed.
    type: bool
    default: false

  file_type:
    description:
      - Type of files to remove.
    type: str
    choices:
      file: remove only regular files.
      link: remove only symbolic links.
      any: remove both files and symbolic links.
    default: file

notes:
  - Directories are never removed by this module, only files and optionally symbolic links.
  - This module will not follow symbolic links when O(recursive=true).
  - Be careful with patterns that might match many files, especially with O(recursive=true).
"""

EXAMPLES = r"""
- name: Remove all log files from /var/log
  community.general.file_remove:
    path: /var/log
    pattern: "*.log"

- name: Remove all temporary files recursively
  community.general.file_remove:
    path: /tmp/myapp
    pattern: "*.tmp"
    recursive: true

- name: Remove files matching a regex pattern
  community.general.file_remove:
    path: /data/backups
    pattern: "backup_[0-9]{8}\\.tar\\.gz"
    use_regex: true

- name: Remove both files and symbolic links
  community.general.file_remove:
    path: /opt/app/cache
    pattern: "cache_*"
    file_type: any

- name: Remove all files starting with 'test_' (check mode)
  community.general.file_remove:
    path: /home/user/tests
    pattern: "test_*"
  check_mode: true
"""

RETURN = r"""
removed_files:
  description: List of files that were removed.
  type: list
  elements: str
  returned: always
  sample: ['/var/log/app.log', '/var/log/error.log']

files_count:
  description: Number of files removed.
  type: int
  returned: success
  sample: 2

path:
  description: The directory path that was searched.
  type: str
  returned: success
  sample: /var/log
"""


import glob
import os
import re
import typing as t

from ansible.module_utils.basic import AnsibleModule


def find_matching_files(
    path: str, pattern: str, use_regex: bool, recursive: bool, file_type: t.Literal["file", "link", "any"]
) -> list[str]:
    """Find all files matching the pattern in the given path."""
    matching_files = []

    if use_regex:
        # Use regular expression matching
        regex = re.compile(pattern)
        if recursive:
            for root, _dirs, files in os.walk(path, followlinks=False):
                for filename in files:
                    if regex.match(filename) or regex.search(filename):
                        full_path = os.path.join(root, filename)
                        if should_include_file(full_path, file_type):
                            matching_files.append(full_path)
        else:
            for filename in os.listdir(path):
                if regex.match(filename) or regex.search(filename):
                    full_path = os.path.join(path, filename)
                    if should_include_file(full_path, file_type):
                        matching_files.append(full_path)
    else:
        # Use glob pattern matching
        if recursive:
            glob_pattern = os.path.join(path, "**", pattern)
            matching_files = [f for f in glob.glob(glob_pattern, recursive=True) if should_include_file(f, file_type)]
        else:
            glob_pattern = os.path.join(path, pattern)
            matching_files = [f for f in glob.glob(glob_pattern) if should_include_file(f, file_type)]

    return sorted(matching_files)


def should_include_file(file_path: str, file_type: t.Literal["file", "link", "any"]) -> bool:
    """Determine if a file should be included based on its type."""
    # Never include directories
    if os.path.isdir(file_path):
        return False

    is_link = os.path.islink(file_path)
    is_file = os.path.isfile(file_path)

    if file_type == "file":
        return is_file and not is_link
    elif file_type == "link":
        return is_link
    else:
        return is_file or is_link


def remove_files(module: AnsibleModule, files: list[str]) -> tuple[list[str], list[tuple[str, str]]]:
    """Remove the specified files and return results."""
    removed_files = []
    failed_files = []

    for file_path in files:
        try:
            if module.check_mode:
                # In check mode, just verify the file exists
                if os.path.exists(file_path):
                    removed_files.append(file_path)
                else:
                    raise FileNotFoundError()
            else:
                # Actually remove the file
                os.remove(file_path)
                removed_files.append(file_path)
        except FileNotFoundError:
            module.warn(f"File not found (likely removed by other process): {file_path}")
        except OSError as e:
            failed_files.append((file_path, str(e)))

    return removed_files, failed_files


def main() -> None:
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type="path", required=True),
            pattern=dict(type="str", required=True),
            use_regex=dict(type="bool", default=False),
            recursive=dict(type="bool", default=False),
            file_type=dict(type="str", default="file", choices=["file", "link", "any"]),
        ),
        supports_check_mode=True,
    )

    path: str = module.params["path"]
    pattern: str = module.params["pattern"]
    use_regex: bool = module.params["use_regex"]
    recursive: bool = module.params["recursive"]
    file_type: t.Literal["file", "link", "any"] = module.params["file_type"]

    # Validate that the path exists and is a directory
    if not os.path.exists(path):
        module.fail_json(msg=f"Path does not exist: {path}")

    if not os.path.isdir(path):
        module.fail_json(msg=f"Path is not a directory: {path}")

    # Validate regex pattern if use_regex is true
    if use_regex:
        try:
            re.compile(pattern)
        except re.error as e:
            module.fail_json(msg=f"Invalid regular expression pattern: {e}")

    # Find matching files
    try:
        matching_files = find_matching_files(path, pattern, use_regex, recursive, file_type)
    except OSError as e:
        module.fail_json(msg=str(e))

    # Prepare diff information
    diff = dict(before=dict(files=matching_files), after=dict(files=[]))

    # Remove the files
    removed_files, failed_files = remove_files(module, matching_files)

    # Prepare result
    changed = len(removed_files) > 0

    result = dict(
        changed=changed,
        removed_files=removed_files,
        files_count=len(removed_files),
        path=path,
        msg=f"Removed {len(removed_files)} file(s) matching pattern '{pattern}'",
    )

    # Add diff if in diff mode
    if module._diff:
        result["diff"] = diff

    # Report any failures
    if failed_files:
        failure_msg = "; ".join([f"{f}: {e}" for f, e in failed_files])
        module.fail_json(
            msg=f"Failed to remove some files: {failure_msg}",
            removed_files=removed_files,
            failed_files=[f for f, e in failed_files],
        )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2020 David Duque Hernández, (@next-davidduquehernandez)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
module: azure_rm_cognitivesearch_info
version_added: "1.4.0"
short_description: Get Azure Cognitive Search service info
description:
    - Get info for a specific Azure Cognitive Search service or all Azure Cognitive Search service within a resource group.

options:
    resource_group:
        description:
            - The name of the Azure resource group.
        type: str
    name:
        description:
            - The name of the Azure Cognitive Search service.
        type: str
    show_keys:
        description:
            - Retrieve admin and query keys.
        type: bool
        default: False

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - David Duque Hernández (@next-davidduquehernandez)
'''

EXAMPLES = '''
- name: Get Azure Cognitive Search info from resource group 'myResourceGroup' and name 'myAzureSearch'
  azure_rm_cognitivesearch_info:
    resource_group: myResourceGroup
    name: myAzureSearch

- name: Get Azure Cognitive Search info from resource group 'myResourceGroup'
  azure_rm_cognitivesearch_info:
    resource_group: myResourceGroup

- name: Get all Azure Cognitive Search info
  azure_rm_cognitivesearch_info:
'''

RETURN = '''
search:
    description:
        - Info for Azure Cognitive Search.
    returned: always
    type: list
    contains:
        hosting_mode:
            description:
                - Type of hosting mode selected.
            returned: always
            type: str
            sample: default
        id:
            description:
                - The unique identifier associated with this Azure Cognitive Search.
            returned: always
            type: str
            sample: /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}
        identity:
            description:
                - The identity of the Azure Cognitive Search Service.
            returned: always
            type: dict
            contains:
                principal_id:
                    description:
                        - Identifier assigned.
                    type: str
                    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
                type:
                    description:
                        - Identity type.
                    returned: always
                    type: str
                    sample: SystemAssigned
            sample:
                principal_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
                type: SystemAssigned
        search_keys:
            description:
                - Admin and query keys for Azure Cognitive Search Service.
            type: dict
            contains:
                admin_primary:
                    description:
                        - Primary admin key for Azure Cognitive Search Service.
                    type: str
                    sample: 12345ABCDE67890FGHIJ123ABC456DEF
                admin_secondary:
                    description:
                        - Secondary admin key for Azure Cognitive Search Service.
                    type: str
                    sample: 12345ABCDE67890FGHIJ123ABC456DEF
                query:
                    description:
                        - List of query keys for Azure Cognitive Search Service.
                    type: list
                    sample: [{'key': '12345ABCDE67890FGHIJ123ABC456DEF', 'name': 'Query key'}]
        keys:
            description:
                - Admin and query keys for Azure Cognitive Search Service.
                - This return value has been deprecated and will be removed in a release after
                  2027-05-01. Use C(search_keys) instead.
            type: dict
            contains:
                admin_primary:
                    description:
                        - Primary admin key for Azure Cognitive Search Service.
                    type: str
                    sample: 12345ABCDE67890FGHIJ123ABC456DEF
                admin_secondary:
                    description:
                        - Secondary admin key for Azure Cognitive Search Service.
                    type: str
                    sample: 12345ABCDE67890FGHIJ123ABC456DEF
                query:
                    description:
                        - List of query keys for Azure Cognitive Search Service.
                    type: list
                    sample: [{'key': '12345ABCDE67890FGHIJ123ABC456DEF', 'name': 'Query key'}]
        location:
            description:
                - The geo-location where the Azure Cognitive Search Service lives.
            returned: always
            type: str
            sample: West Europe
        name:
            description:
                - The name of the Azure Cognitive Search Service.
            returned: always
            type: str
            sample: myazuresearch
        network_rule_set:
            description:
                - Network specific rules that determine how the Azure Cognitive Search service may be reached.
            returned: always
            type: list
            sample: ['1.1.1.1', '8.8.8.8/31']
        partition_count:
            description:
                - The number of partitions in the Azure Cognitive Search Service.
            returned: always
            type: int
            sample: 3
        provisioning_state:
            description:
                - The state of the provisioning state of Azure Cognitive Search Service.
            returned: always
            type: str
            sample: succeeded
        public_network_access:
            description:
                - If it's allowed traffic over public interface.
            returned: always
            type: str
            sample: enabled
        replica_count:
            description:
                - The number of replicas in the Azure Cognitive Search Service.
            returned: always
            type: int
            sample: 3
        sku:
            description:
                - The SKU of the Azure Cognitive Search Service.
            returned: always
            type: str
            sample: standard
        status:
            description:
                - The state of the Azure Cognitive Search.
            returned: always
            type: str
            sample: Active running
        tags:
            description:
                - The resource tags.
            returned: always
            type: dict
            sample: { "tag1":"abc" }
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMSearchInfo(AzureRMModuleBase):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            show_keys=dict(type='bool', default=False)
        )

        self.results = dict(
            changed=False,
            search=[]
        )

        self.name = None
        self.resource_group = None
        self.show_keys = False

        super(AzureRMSearchInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                supports_check_mode=True,
                                                supports_tags=False)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.show_keys:
            self.module.deprecate(
                "The 'keys' return key is deprecated. Use 'search_keys' instead.",
                date="2027-05-01",
                collection_name="azure.azcollection",
            )

        if self.name and not self.resource_group:
            self.fail("Parameter error: resource group required when filtering by name.")

        results = []
        if self.name:
            results = self.get_search()
        elif self.resource_group:
            results = self.list_resource_group()
        else:
            results = self.list_all()

        self.results['search'] = results
        return self.results

    def get_search(self):
        self.log('Get properties for azure search {0}'.format(self.name))
        search_obj = None
        account_dict = None

        try:
            search_obj = self.search_client.services.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass

        if search_obj:
            account_dict = self.account_obj_to_dict(search_obj)

        return account_dict

    def list_resource_group(self):
        self.log('Get basic properties for azure search in resource group {0}'.format(self.resource_group))
        search_obj = None
        results = list()

        try:
            search_obj = self.search_client.services.list_by_resource_group(self.resource_group)
        except Exception:
            pass

        if search_obj:
            for seach_item in search_obj:
                results.append(self.account_obj_to_dict(seach_item))
            return results

        return list()

    def list_all(self):
        self.log('Get basic properties for all azure search')
        search_obj = None
        results = list()

        try:
            search_obj = self.search_client.services.list_by_subscription()
        except Exception:
            pass

        if search_obj:
            for search_item in search_obj:
                results.append(self.account_obj_to_dict(search_item))
            return results

        return list()

    def account_obj_to_dict(self, search_obj):
        account_dict = dict(
            hosting_mode=search_obj.hosting_mode,
            id=search_obj.id,
            identity=dict(type=search_obj.identity.type if search_obj.identity else 'None'),
            location=search_obj.location,
            name=search_obj.name,
            network_rule_set=list(),
            partition_count=search_obj.partition_count,
            provisioning_state=search_obj.provisioning_state,
            public_network_access=search_obj.public_network_access,
            replica_count=search_obj.replica_count,
            sku=search_obj.sku.name,
            status=search_obj.status,
            tags=search_obj.tags
        )

        if search_obj.identity and search_obj.identity.principal_id:
            account_dict['identity']['principal_id'] = search_obj.identity.principal_id

        for rule in search_obj.network_rule_set.ip_rules:
            account_dict['network_rule_set'].append(rule.value)

        if self.show_keys:
            account_dict['search_keys'] = dict()

            admin_keys = self.search_client.admin_keys.get(self.resource_group, self.name)
            account_dict['search_keys']['admin_primary'] = admin_keys.primary_key
            account_dict['search_keys']['admin_secondary'] = admin_keys.secondary_key

            query_keys = self.search_client.query_keys.list_by_search_service(self.resource_group, self.name)
            account_dict['search_keys']['query'] = list()
            for key in query_keys:
                account_dict['search_keys']['query'].append(dict(name=key.name, key=key.key))

            account_dict['keys'] = account_dict['search_keys']

        return account_dict


def main():
    AzureRMSearchInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_add_outbound_inspection_certificate
short_description: Add outbound-inspection-certificate
description:
  - Add outbound-inspection-certificate
  - All operations are performed over Web Services API.
  - Available from R81.20 management version.
version_added: "6.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  issued_by:
    description:
      - The DN (Distinguished Name) of the certificate.
    type: str
    required: True
  base64_password:
    description:
      - Password (encoded in Base64 with padding) for the certificate file.
    type: str
    required: True
  valid_from:
    description:
      - The date, from which the certificate is valid. Format, YYYY-MM-DD.
    type: str
    required: True
  valid_to:
    description:
      - The certificate expiration date. Format, YYYY-MM-DD.
    type: str
    required: True
  name:
    description:
      - Object name.
      - Available from R82 management version.
    type: str
  is_default:
    description:
      - Is the certificate the default certificate.
      - Available from R82 management version.
    type: bool
  tags:
    description:
      - Collection of tag identifiers.
      - Available from R82 management version.
    type: list
    elements: str
  color:
    description:
      - Color of the object. Should be one of existing colors.
      - Available from R82 management version.
    type: str
    choices: ['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green', 'khaki', 'orchid', 'dark orange', 'dark sea green',
             'pink', 'turquoise', 'dark blue', 'firebrick', 'brown', 'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon',
             'coral', 'sea green', 'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna', 'yellow']
  comments:
    description:
      - Comments string.
      - Available from R82 management version.
    type: str
  details_level:
    description:
      - The level of detail for some of the fields in the response can vary from showing only the UID value of the object to a fully detailed
        representation of the object.
    type: str
    choices: ['uid', 'standard', 'full']
  ignore_warnings:
    description:
      - Apply changes ignoring warnings.
      - Available from R82 management version.
    type: bool
  ignore_errors:
    description:
      - Apply changes ignoring errors. You won't be able to publish such a changes. If ignore-warnings flag was omitted - warnings will also be ignored.
      - Available from R82 management version.
    type: bool
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: add-outbound-inspection-certificate
  cp_mgmt_add_outbound_inspection_certificate:
    base64_password: bXlfcGFzc3dvcmQ=
    is_default: 'false'
    issued_by: www.checkpoint.com
    name: OutboundCertificate
    valid_from: '2021-04-17'
    valid_to: '2028-04-17'
"""

RETURN = """
cp_mgmt_add_outbound_inspection_certificate:
  description: The checkpoint add-outbound-inspection-certificate output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        issued_by=dict(type='str', required=True),
        base64_password=dict(type='str', no_log=True, required=True),
        valid_from=dict(type='str', required=True),
        valid_to=dict(type='str', required=True),
        name=dict(type='str'),
        is_default=dict(type='bool'),
        tags=dict(type='list', elements='str'),
        color=dict(type='str', choices=['aquamarine', 'black', 'blue', 'crete blue', 'burlywood', 'cyan', 'dark green',
                                        'khaki', 'orchid', 'dark orange', 'dark sea green', 'pink', 'turquoise', 'dark blue', 'firebrick', 'brown',
                                        'forest green', 'gold', 'dark gold', 'gray', 'dark gray', 'light green', 'lemon chiffon', 'coral', 'sea green',
                                        'sky blue', 'magenta', 'purple', 'slate blue', 'violet red', 'navy blue', 'olive', 'orange', 'red', 'sienna',
                                        'yellow']),
        comments=dict(type='str'),
        details_level=dict(type='str', choices=['uid', 'standard', 'full']),
        ignore_warnings=dict(type='bool'),
        ignore_errors=dict(type='bool')
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)
    command = 'add-outbound-inspection-certificate'

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos ntp_global fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import utils

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.ntp_global.ntp_global import (
    Ntp_globalArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.ntp_global import (
    Ntp_globalTemplate,
)


class Ntp_globalFacts(object):
    """The nxos ntp_global facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = Ntp_globalArgs.argument_spec

    def get_config(self, connection):
        """Wrapper method for `connection.get()`
        This method exists solely to allow the unit test framework to mock device connection calls.
        """
        return connection.get("show running-config ntp")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Ntp_global network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self.get_config(connection)

        # parse native config using the Ntp_global template
        ntp_global_parser = Ntp_globalTemplate(lines=data.splitlines(), module=self._module)
        objs = ntp_global_parser.parse()

        if "access_group" in objs:
            for x in ["peer", "query_only", "serve", "serve_only"]:
                if x in objs["access_group"]:
                    objs["access_group"][x] = sorted(
                        objs["access_group"][x],
                        key=lambda k: k["access_list"],
                    )

        pkey = {
            "authentication_keys": "id",
            "peers": "peer",
            "servers": "server",
            "trusted_keys": "key_id",
        }

        for x in pkey.keys():
            if x in objs:
                objs[x] = sorted(objs[x], key=lambda k: k[pkey[x]])

        ansible_facts["ansible_network_resources"].pop("ntp_global", None)

        params = utils.remove_empties(
            ntp_global_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )

        facts["ntp_global"] = params.get("config", {})
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python

# Copyright (c) 2016, Adam Števko <adam.stevko@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: beadm
short_description: Manage ZFS boot environments on FreeBSD/Solaris/illumos systems
description:
  - Create, delete or activate ZFS boot environments.
  - Mount and unmount ZFS boot environments.
author: Adam Števko (@xen0l)
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    description:
      - ZFS boot environment name.
    type: str
    required: true
    aliases: ["be"]
  snapshot:
    description:
      - If specified, the new boot environment is cloned from the given snapshot or inactive boot environment.
    type: str
  description:
    description:
      - Associate a description with a new boot environment. This option is available only on Solarish platforms.
    type: str
  options:
    description:
      - Create the datasets for new BE with specific ZFS properties.
      - Multiple options can be specified.
      - This option is available only on Solarish platforms.
    type: str
  mountpoint:
    description:
      - Path where to mount the ZFS boot environment.
    type: path
  state:
    description:
      - Create or delete ZFS boot environment.
    type: str
    choices: [absent, activated, mounted, present, unmounted]
    default: present
  force:
    description:
      - Specifies if the unmount should be forced.
    type: bool
    default: false
"""

EXAMPLES = r"""
- name: Create ZFS boot environment
  community.general.beadm:
    name: upgrade-be
    state: present

- name: Create ZFS boot environment from existing inactive boot environment
  community.general.beadm:
    name: upgrade-be
    snapshot: be@old
    state: present

- name: Create ZFS boot environment with compression enabled and description "upgrade"
  community.general.beadm:
    name: upgrade-be
    options: "compression=on"
    description: upgrade
    state: present

- name: Delete ZFS boot environment
  community.general.beadm:
    name: old-be
    state: absent

- name: Mount ZFS boot environment on /tmp/be
  community.general.beadm:
    name: BE
    mountpoint: /tmp/be
    state: mounted

- name: Unmount ZFS boot environment
  community.general.beadm:
    name: BE
    state: unmounted

- name: Activate ZFS boot environment
  community.general.beadm:
    name: upgrade-be
    state: activated
"""

RETURN = r"""
name:
  description: BE name.
  returned: always
  type: str
  sample: pre-upgrade
snapshot:
  description: ZFS snapshot to create BE from.
  returned: always
  type: str
  sample: rpool/ROOT/oi-hipster@fresh
description:
  description: BE description.
  returned: always
  type: str
  sample: Upgrade from 9.0 to 10.0
options:
  description: BE additional options.
  returned: always
  type: str
  sample: compression=on
mountpoint:
  description: BE mountpoint.
  returned: always
  type: str
  sample: /mnt/be
state:
  description: State of the target.
  returned: always
  type: str
  sample: present
force:
  description: If forced action is wanted.
  returned: always
  type: bool
  sample: false
"""

import os

from ansible.module_utils.basic import AnsibleModule


class BE:
    def __init__(self, module):
        self.module = module

        self.name = module.params["name"]
        self.snapshot = module.params["snapshot"]
        self.description = module.params["description"]
        self.options = module.params["options"]
        self.mountpoint = module.params["mountpoint"]
        self.state = module.params["state"]
        self.force = module.params["force"]
        self.is_freebsd = os.uname()[0] == "FreeBSD"

    def _beadm_list(self):
        cmd = [self.module.get_bin_path("beadm"), "list", "-H"]
        if "@" in self.name:
            cmd.append("-s")
        return self.module.run_command(cmd)

    def _find_be_by_name(self, out):
        if "@" in self.name:
            for line in out.splitlines():
                if self.is_freebsd:
                    check = line.split()
                    if check == []:
                        continue
                    full_name = check[0].split("/")
                    if full_name == []:
                        continue
                    check[0] = full_name[len(full_name) - 1]
                    if check[0] == self.name:
                        return check
                else:
                    check = line.split(";")
                    if check[0] == self.name:
                        return check
        else:
            for line in out.splitlines():
                if self.is_freebsd:
                    check = line.split()
                    if check[0] == self.name:
                        return check
                else:
                    check = line.split(";")
                    if check[0] == self.name:
                        return check
        return None

    def exists(self):
        (rc, out, dummy) = self._beadm_list()

        if rc == 0:
            return bool(self._find_be_by_name(out))
        else:
            return False

    def is_activated(self):
        (rc, out, dummy) = self._beadm_list()

        if rc == 0:
            line = self._find_be_by_name(out)
            if line is None:
                return False
            if self.is_freebsd:
                if "R" in line[1]:
                    return True
            else:
                if "R" in line[2]:
                    return True

        return False

    def activate_be(self):
        cmd = [self.module.get_bin_path("beadm"), "activate", self.name]
        return self.module.run_command(cmd)

    def create_be(self):
        cmd = [self.module.get_bin_path("beadm"), "create"]

        if self.snapshot:
            cmd.extend(["-e", self.snapshot])
        if not self.is_freebsd:
            if self.description:
                cmd.extend(["-d", self.description])
            if self.options:
                cmd.extend(["-o", self.options])

        cmd.append(self.name)

        return self.module.run_command(cmd)

    def destroy_be(self):
        cmd = [self.module.get_bin_path("beadm"), "destroy", "-F", self.name]
        return self.module.run_command(cmd)

    def is_mounted(self):
        (rc, out, dummy) = self._beadm_list()

        if rc == 0:
            line = self._find_be_by_name(out)
            if line is None:
                return False
            if self.is_freebsd:
                # On FreeBSD, we exclude currently mounted BE on /, as it is
                # special and can be activated even if it is mounted. That is not
                # possible with non-root BEs.
                if line[2] != "-" and line[2] != "/":
                    return True
            else:
                if line[3]:
                    return True

        return False

    def mount_be(self):
        cmd = [self.module.get_bin_path("beadm"), "mount", self.name]

        if self.mountpoint:
            cmd.append(self.mountpoint)

        return self.module.run_command(cmd)

    def unmount_be(self):
        cmd = [self.module.get_bin_path("beadm"), "unmount"]
        if self.force:
            cmd.append("-f")
        cmd.append(self.name)

        return self.module.run_command(cmd)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True, aliases=["be"]),
            snapshot=dict(type="str"),
            description=dict(type="str"),
            options=dict(type="str"),
            mountpoint=dict(type="path"),
            state=dict(
                type="str", default="present", choices=["absent", "activated", "mounted", "present", "unmounted"]
            ),
            force=dict(type="bool", default=False),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    be = BE(module)

    rc = None
    out = ""
    err = ""
    result = {}
    result["name"] = be.name
    result["state"] = be.state

    if be.snapshot:
        result["snapshot"] = be.snapshot

    if be.description:
        result["description"] = be.description

    if be.options:
        result["options"] = be.options

    if be.mountpoint:
        result["mountpoint"] = be.mountpoint

    if be.state == "absent":
        # beadm on FreeBSD and Solarish systems differs in delete behaviour in
        # that we are not allowed to delete activated BE on FreeBSD while on
        # Solarish systems we cannot delete BE if it is mounted. We add mount
        # check for both platforms as BE should be explicitly unmounted before
        # being deleted. On FreeBSD, we also check if the BE is activated.
        if be.exists():
            if not be.is_mounted():
                if module.check_mode:
                    module.exit_json(changed=True)

                if be.is_freebsd:
                    if be.is_activated():
                        module.fail_json(msg="Unable to remove active BE!")

                (rc, out, err) = be.destroy_be()

                if rc != 0:
                    module.fail_json(msg=f'Error while destroying BE: "{err}"', name=be.name, stderr=err, rc=rc)
            else:
                module.fail_json(msg="Unable to remove BE as it is mounted!")

    elif be.state == "present":
        if not be.exists():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.create_be()

            if rc != 0:
                module.fail_json(msg=f'Error while creating BE: "{err}"', name=be.name, stderr=err, rc=rc)

    elif be.state == "activated":
        if not be.is_activated():
            if module.check_mode:
                module.exit_json(changed=True)

            # On FreeBSD, beadm is unable to activate mounted BEs, so we add
            # an explicit check for that case.
            if be.is_freebsd:
                if be.is_mounted():
                    module.fail_json(msg="Unable to activate mounted BE!")

            (rc, out, err) = be.activate_be()

            if rc != 0:
                module.fail_json(msg=f'Error while activating BE: "{err}"', name=be.name, stderr=err, rc=rc)
    elif be.state == "mounted":
        if not be.is_mounted():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.mount_be()

            if rc != 0:
                module.fail_json(msg=f'Error while mounting BE: "{err}"', name=be.name, stderr=err, rc=rc)

    elif be.state == "unmounted":
        if be.is_mounted():
            if module.check_mode:
                module.exit_json(changed=True)

            (rc, out, err) = be.unmount_be()

            if rc != 0:
                module.fail_json(msg=f'Error while unmounting BE: "{err}"', name=be.name, stderr=err, rc=rc)

    if rc is None:
        result["changed"] = False
    else:
        result["changed"] = True

    if out:
        result["stdout"] = out
    if err:
        result["stderr"] = err

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_aaa_custom_privilege
short_description: Manage AAA RBAC Custom Privileges (aaa:RbacClassPriv)
description:
- Manage AAA Custom Privileges with RBAC Rules on Cisco ACI fabrics.
options:
  name:
    description:
    - Name of the object class for which you are configuring access.
    type: str
    aliases: [ custom_privilege_name ]
  description:
    description:
    - Description of the AAA custom privilege.
    type: str
    aliases: [ descr ]
  write_privilege:
    description:
    - Name of the custom privilege that will include write access to objects of the class.
    type: str
    aliases: [ write_priv, w_priv ]
    choices: [
      custom-privilege-1,
      custom-privilege-2,
      custom-privilege-3,
      custom-privilege-4,
      custom-privilege-5,
      custom-privilege-6,
      custom-privilege-7,
      custom-privilege-8,
      custom-privilege-9,
      custom-privilege-10,
      custom-privilege-11,
      custom-privilege-12,
      custom-privilege-13,
      custom-privilege-14,
      custom-privilege-15,
      custom-privilege-16,
      custom-privilege-17,
      custom-privilege-18,
      custom-privilege-19,
      custom-privilege-20,
      custom-privilege-21,
      custom-privilege-22
    ]
  read_privilege:
    description:
    - Name of the custom privilege that will include read access to objects of the class.
    type: str
    aliases: [ read_priv, r_priv ]
    choices: [
      custom-privilege-1,
      custom-privilege-2,
      custom-privilege-3,
      custom-privilege-4,
      custom-privilege-5,
      custom-privilege-6,
      custom-privilege-7,
      custom-privilege-8,
      custom-privilege-9,
      custom-privilege-10,
      custom-privilege-11,
      custom-privilege-12,
      custom-privilege-13,
      custom-privilege-14,
      custom-privilege-15,
      custom-privilege-16,
      custom-privilege-17,
      custom-privilege-18,
      custom-privilege-19,
      custom-privilege-20,
      custom-privilege-21,
      custom-privilege-22
    ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
  name_alias:
    description:
    - The alias for the current object. This relates to the nameAlias field in ACI.
    type: str
extends_documentation_fragment:
- cisco.aci.aci

seealso:
- module: cisco.aci.aci_aaa_domain
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(aaa:Domain).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Sabari Jaganathan (@sajagana)
"""

EXAMPLES = r"""
- name: Add a custom privilege
  cisco.aci.aci_aaa_custom_privilege:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: fabricPod
    write_privilege: custom-privilege-1
    read_privilege: custom-privilege-1
    state: present
  delegate_to: localhost

- name: Add list of custom privileges
  cisco.aci.aci_aaa_custom_privilege:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: "{{ item.name }}"
    write_privilege: "{{ item.write_privilege }}"
    read_privilege: "{{ item.read_privilege | default('') }}"
    state: present
  with_items:
    - name: fvTenant
      write_privilege: custom-privilege-2
      read_privilege: custom-privilege-2
    - name: aaaUser
      write_privilege: custom-privilege-3
  delegate_to: localhost

- name: Query a custom privilege with name
  cisco.aci.aci_aaa_custom_privilege:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: fabricPod
    state: query
  delegate_to: localhost

- name: Query all custom privileges
  cisco.aci.aci_aaa_custom_privilege:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost

- name: Remove a custom privilege
  cisco.aci.aci_aaa_custom_privilege:
    host: apic
    username: admin
    password: SomeSecretPassword
    name: fabricPod
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: '?rsp-prop-include=config-only'
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec

CUSTOM_PRIVILEGES = [
    "custom-privilege-1",
    "custom-privilege-2",
    "custom-privilege-3",
    "custom-privilege-4",
    "custom-privilege-5",
    "custom-privilege-6",
    "custom-privilege-7",
    "custom-privilege-8",
    "custom-privilege-9",
    "custom-privilege-10",
    "custom-privilege-11",
    "custom-privilege-12",
    "custom-privilege-13",
    "custom-privilege-14",
    "custom-privilege-15",
    "custom-privilege-16",
    "custom-privilege-17",
    "custom-privilege-18",
    "custom-privilege-19",
    "custom-privilege-20",
    "custom-privilege-21",
    "custom-privilege-22",
]


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        name=dict(type="str", aliases=["custom_privilege_name"]),
        description=dict(type="str", aliases=["descr"]),
        write_privilege=dict(
            type="str",
            aliases=["write_priv", "w_priv"],
            choices=CUSTOM_PRIVILEGES,
        ),
        read_privilege=dict(
            type="str",
            aliases=["read_priv", "r_priv"],
            choices=CUSTOM_PRIVILEGES,
        ),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        name_alias=dict(type="str"),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name"]],
        ],
    )

    aci = ACIModule(module)

    name = module.params.get("name")
    description = module.params.get("description")
    w_priv = module.params.get("write_privilege")
    r_priv = module.params.get("read_privilege")
    state = module.params.get("state")
    name_alias = module.params.get("name_alias")

    aci.construct_url(
        root_class=dict(aci_class="aaaRbacClassPriv", aci_rn="rbacdb/rbacclpriv-{0}".format(name), module_object=name, target_filter=dict(name=name)),
    )
    aci.get_existing()

    if state == "present":
        aci.payload(
            aci_class="aaaRbacClassPriv",
            class_config=dict(
                name=name,
                descr=description,
                wPriv=w_priv,
                rPriv=r_priv,
                nameAlias=name_alias,
            ),
        )

        aci.get_diff(aci_class="aaaRbacClassPriv")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2025 Klaas Demter (@Klaas-)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_monitoractivitylogalerts_info
version_added: "3.7.0"
short_description: Get Activity log alerts
description:
    - Get Activity log alerts

options:
    name:
        description:
            - The name of the activity log alert you're trying to get details about.
        type: str
    resource_group:
        description:
            - The name of the resource group in which the activity log alert is (if you use name)
            - The name of the resource group where you want to list Activity log alerts (if you don't use name)
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Klaas Demter (@Klaas-)
'''

EXAMPLES = '''
- name: Get activity log alert details
  azure.azcollection.azure_rm_monitoractivitylogalerts_info:
    name: activity_log_alert_name
    resource_group: resource_group_name

- name: Get all Activity log alerts in specific resource group
  azure.azcollection.azure_rm_monitoractivitylogalerts_info:
    resource_group: resource_group_name

- name: Get all Activity log alerts in the current subscription
  azure.azcollection.azure_rm_monitoractivitylogalerts_info:
'''

RETURN = '''
activitylogalerts:
    description:
        - List of Activity log alerts
        - Can be empty if listing Activity log alerts
    type: list
    returned: always
    sample: [
        {
            "actions": {
                "action_groups": [
                    {
                        "action_group_id": \
"/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resource_group_name/providers/Microsoft.Insights/actionGroups/action_group_name",
                        "webhook_properties": {}
                    }
                ]
            },
            "condition": {
                "all_of": [
                    {
                        "equals": "ResourceHealth",
                        "field": "category"
                    },
                    {
                        "any_of": [
                            {
                                "equals": "Unknown",
                                "field": "properties.currentHealthStatus"
                            },
                            {
                                "equals": "Unavailable",
                                "field": "properties.currentHealthStatus"
                            },
                            {
                                "equals": "Degraded",
                                "field": "properties.currentHealthStatus"
                            }
                        ]
                    },
                    {
                        "any_of": [
                            {
                                "equals": "Unknown",
                                "field": "properties.previousHealthStatus"
                            },
                            {
                                "equals": "Unavailable",
                                "field": "properties.previousHealthStatus"
                            },
                            {
                                "equals": "Degraded",
                                "field": "properties.previousHealthStatus"
                            },
                            {
                                "equals": "Available",
                                "field": "properties.previousHealthStatus"
                            }
                        ]
                    },
                    {
                        "any_of": [
                            {
                                "equals": "PlatformInitiated",
                                "field": "properties.cause"
                            }
                        ]
                    }
                ]
            },
            "description": "Good description of the alert",
            "enabled": true,
            "id": \
"/subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resource_group_name/providers/Microsoft.Insights/activityLogAlerts/activity_log_alert_name",
            "location": "global",
            "name": "activity_log_alert_name",
            "scopes": [
                "/subscriptions/xxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/resource_group_name"
            ],
            "tags": {},
            "type": "Microsoft.Insights/ActivityLogAlerts"
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError, HttpResponseError

except ImportError:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'activitylogalerts'


class AzureRMactivitylogalertsInfo(AzureRMModuleBase):
    """Information class for an Azure RM Activity log alerts"""

    def __init__(self):
        self.module_arg_spec = dict(
            name=dict(type='str', required=False),
            resource_group=dict(type='str', required=False),
        )

        self.required_by = {
            'name': 'resource_group'
        }

        self.name = None
        self.resource_group = None
        self.log_path = None
        self.log_mode = None

        self.results = dict(
            changed=False,
            activitylogalerts=[]
        )

        super(AzureRMactivitylogalertsInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                           supports_check_mode=True,
                                                           supports_tags=False,
                                                           facts_module=True,
                                                           required_by=self.required_by)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name:
            result = self.get_activity_log_alert()
        else:
            result = self.list_activity_log_alerts()

        self.results['activitylogalerts'] = result

        return self.results

    def get_activity_log_alert(self):
        '''
        Gets the properties of the specified activity log alert.

        :return: List of Activity log alerts
        '''
        self.log("Checking if activity log alert {0} in resource group {1} is present".format(self.name,
                                                                                              self.resource_group))

        result = []
        activity_log_alert = None

        try:
            activity_log_alert = self.monitor_management_client_activity_log_alerts.activity_log_alerts.get(activity_log_alert_name=self.name,
                                                                                                            resource_group_name=self.resource_group)
        except ResourceNotFoundError as ex:
            self.log("Could not find activity log alert {0} in resource group {1}".format(self.name, self.resource_group))
            return []
        except HttpResponseError as ex:
            if ex.error.code == 'InvalidSubscriptionId':
                self.log("Could not find subscription id")
                return []
            else:
                raise Exception(ex)
        if activity_log_alert:
            result = [self.serialize_obj(activity_log_alert, AZURE_OBJECT_CLASS)]

        return result

    def list_activity_log_alerts(self):
        '''
        Gets the properties of the specified Activity log alerts in resource group or subscription.

        :return: List of Activity log alerts
        '''

        result = []
        activity_log_alerts = None

        if self.resource_group:
            self.log("Checking if the Activity log alerts in resource group {0} are present".format(self.resource_group))
            activity_log_alerts_mgmt_client = self.monitor_management_client_activity_log_alerts.activity_log_alerts
            activity_log_alerts = activity_log_alerts_mgmt_client.list_by_resource_group(resource_group_name=self.resource_group)
        else:
            self.log("Checking if the Activity log alerts are present in subscription")
            activity_log_alerts = self.monitor_management_client_activity_log_alerts.activity_log_alerts.list_by_subscription_id()
        if activity_log_alerts:
            # it seems the exception is thrown when iterating through activity_log_alerts, not when setting activity_log_alerts
            try:
                for item in activity_log_alerts:
                    result.append(self.serialize_obj(item, AZURE_OBJECT_CLASS))
            except ResourceNotFoundError as ex:
                self.log("Could not find resource group {0}".format(self.resource_group))
            except HttpResponseError as ex:
                if ex.error.code == 'InvalidSubscriptionId':
                    self.log("Could not find subscription id")
                else:
                    raise Exception(ex)

        return result


def main():
    """Main execution"""
    AzureRMactivitylogalertsInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2020, quidame <quidame@poivron.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import time
import typing as t

from ansible.errors import AnsibleActionFail, AnsibleConnectionFailure
from ansible.plugins.action import ActionBase
from ansible.utils.display import Display
from ansible.utils.vars import merge_hash

display = Display()


class ActionModule(ActionBase):
    # Keep internal params away from user interactions
    _VALID_ARGS = frozenset(("path", "state", "table", "noflush", "counters", "modprobe", "ip_version", "wait"))
    DEFAULT_SUDOABLE = True

    @staticmethod
    def msg_error__async_and_poll_not_zero(task_poll, task_async, max_timeout) -> str:
        return (
            "This module doesn't support async>0 and poll>0 when its 'state' param "
            "is set to 'restored'. To enable its rollback feature (that needs the "
            "module to run asynchronously on the remote), please set task attribute "
            f"'poll' (={task_poll}) to 0, and 'async' (={task_async}) to a value >2 and not greater than "
            f"'ansible_timeout' (={max_timeout}) (recommended)."
        )

    @staticmethod
    def msg_warning__no_async_is_no_rollback(task_poll, task_async, max_timeout) -> str:
        return (
            "Attempts to restore iptables state without rollback in case of mistake "
            "may lead the ansible controller to loose access to the hosts and never "
            "regain it before fixing firewall rules through a serial console, or any "
            f"other way except SSH. Please set task attribute 'poll' (={task_poll}) to 0, and "
            f"'async' (={task_async}) to a value >2 and not greater than 'ansible_timeout' (={max_timeout}) "
            "(recommended)."
        )

    @staticmethod
    def msg_warning__async_greater_than_timeout(task_poll, task_async, max_timeout) -> str:
        return (
            "You attempt to restore iptables state with rollback in case of mistake, "
            "but with settings that will lead this rollback to happen AFTER that the "
            "controller will reach its own timeout. Please set task attribute 'poll' "
            f"(={task_poll}) to 0, and 'async' (={task_async}) to a value >2 and not greater than "
            f"'ansible_timeout' (={max_timeout}) (recommended)."
        )

    def _async_result(
        self, async_status_args: dict[str, t.Any], task_vars: dict[str, t.Any], timeout: int
    ) -> dict[str, t.Any]:
        """
        Retrieve results of the asynchronous task, and display them in place of
        the async wrapper results (those with the ansible_job_id key).
        """
        async_status = self._task.copy()
        async_status.args = async_status_args
        async_status.action = "ansible.builtin.async_status"
        async_status.async_val = 0
        async_action = self._shared_loader_obj.action_loader.get(
            async_status.action,
            task=async_status,
            connection=self._connection,
            play_context=self._play_context,
            loader=self._loader,
            templar=self._templar,
            shared_loader_obj=self._shared_loader_obj,
        )

        if async_status.args["mode"] == "cleanup":
            return async_action.run(task_vars=task_vars)

        # At least one iteration is required, even if timeout is 0.
        for dummy in range(max(1, timeout)):
            async_result = async_action.run(task_vars=task_vars)
            if async_result.get("finished", 0) == 1:
                break
            time.sleep(min(1, timeout))

        return async_result

    def run(self, tmp: str | None = None, task_vars: dict[str, t.Any] | None = None) -> dict[str, t.Any]:
        self._supports_check_mode = True
        self._supports_async = True

        if task_vars is None:
            task_vars = {}

        result = super().run(tmp, task_vars)
        del tmp  # tmp no longer has any effect

        if not result.get("skipped"):
            # FUTURE: better to let _execute_module calculate this internally?
            wrap_async = self._task.async_val and not self._connection.has_native_async

            # Set short names for values we'll have to compare or reuse
            task_poll = self._task.poll
            task_async = self._task.async_val
            check_mode = self._play_context.check_mode
            max_timeout = self._connection._play_context.timeout
            module_args = self._task.args

            async_status_args = {}
            starter_cmd = None
            confirm_cmd = None

            if module_args.get("state", None) == "restored":
                if not wrap_async:
                    if not check_mode:
                        display.warning(self.msg_error__async_and_poll_not_zero(task_poll, task_async, max_timeout))
                elif task_poll:
                    raise AnsibleActionFail(
                        self.msg_warning__no_async_is_no_rollback(task_poll, task_async, max_timeout)
                    )
                else:
                    if task_async > max_timeout and not check_mode:
                        display.warning(
                            self.msg_warning__async_greater_than_timeout(task_poll, task_async, max_timeout)
                        )

                    # inject the async directory based on the shell option into the
                    # module args
                    async_dir = self.get_shell_option("async_dir", default="~/.ansible_async")

                    # Bind the loop max duration to consistent values on both
                    # remote and local sides (if not the same, make the loop
                    # longer on the controller); and set a backup file path.
                    module_args["_timeout"] = task_async
                    module_args["_back"] = f"{async_dir}/iptables.state"
                    async_status_args = dict(mode="status")
                    confirm_cmd = f"rm -f {module_args['_back']}"
                    starter_cmd = f"touch {module_args['_back']}.starter"
                    remaining_time = max(task_async, max_timeout)

            # do work!
            result = merge_hash(
                result, self._execute_module(module_args=module_args, task_vars=task_vars, wrap_async=wrap_async)
            )

            # Then the 3-steps "go ahead or rollback":
            # 1. Catch early errors of the module (in asynchronous task) if any.
            #    Touch a file on the target to signal the module to process now.
            # 2. Reset connection to ensure a persistent one will not be reused.
            # 3. Confirm the restored state by removing the backup on the remote.
            #    Retrieve the results of the asynchronous task to return them.
            if starter_cmd is not None and confirm_cmd is not None:
                async_status_args["jid"] = result.get("ansible_job_id", None)
                if async_status_args["jid"] is None:
                    raise AnsibleActionFail("Unable to get 'ansible_job_id'.")

                # Catch early errors due to missing mandatory option, bad
                # option type/value, missing required system command, etc.
                result = merge_hash(result, self._async_result(async_status_args, task_vars, 0))

                # The module is aware to not process the main iptables-restore
                # command before finding (and deleting) the 'starter' cookie on
                # the host, so the previous query will not reach ssh timeout.
                self._low_level_execute_command(starter_cmd, sudoable=self.DEFAULT_SUDOABLE)

                # As the main command is not yet executed on the target, here
                # 'finished' means 'failed before main command be executed'.
                if not result["finished"]:
                    try:
                        self._connection.reset()
                    except AttributeError:
                        pass

                    for dummy in range(max_timeout):
                        time.sleep(1)
                        remaining_time -= 1
                        # - AnsibleConnectionFailure covers rejected requests (i.e.
                        #   by rules with '--jump REJECT')
                        # - ansible_timeout is able to cover dropped requests (due
                        #   to a rule or policy DROP) if not lower than async_val.
                        try:
                            self._low_level_execute_command(confirm_cmd, sudoable=self.DEFAULT_SUDOABLE)
                            break
                        except AnsibleConnectionFailure:
                            continue

                    result = merge_hash(result, self._async_result(async_status_args, task_vars, remaining_time))

                # Cleanup async related stuff and internal params
                for key in ("ansible_job_id", "results_file", "started", "finished"):
                    if result.get(key):
                        del result[key]

                if result.get("invocation", {}).get("module_args"):
                    for key in ("_back", "_timeout", "_async_dir", "jid"):
                        if result["invocation"]["module_args"].get(key):
                            del result["invocation"]["module_args"][key]

                async_status_args["mode"] = "cleanup"
                self._async_result(async_status_args, task_vars, 0)

            if not wrap_async:
                # remove a temporary path we created
                self._remove_tmp_path(self._connection._shell.tmpdir)

        return result
